/* lib/dynscreen80.p
 *
 * Initialization
 * ==============
 * protop.p
 *   -> lib/viewer.p -> lib/dynscreen.p							(actual useful initialization)
 *                        -> dynViewerInit
 *   -> lib/browser.p -> lib/dynscreen.p
 *                         -> dynBrowserInit
 *   -> publish protop_show -> lib/dynscreen.p						(for each startup panel)
 *                               -> protop_show
 *                                    -> protop_display
 *
 *
 *
 * When panels are added or removed
 * ================================
 * protop.p
 *   -> publish protop_show -> lib/dynscreen.p						(for the panel being added or removed)
 *                               -> protop_show
 *                                    -> protop_display
 *   -> refresh ...
 *
 *
 * On refresh if a panel changes size
 * ==================================
 * protop.p
 *   -> publish resizeBrowse -> lib/dynscreen.p						(for the resized browse)
 *                                -> resizeBrowse
 *                                     -> protop_show (published)
 *                                          -> protop_display
 *   -> refresh ...
 *
 *
 * When interval timer expires
 * ===========================
 * protop.p
 *   -> refresh
 *        -> publish protop_getReqStr -> lib/dynscreen.p
 *                                         -> protop_getReqStr
 *        -> makeRequest
 *             -> paintScreen -> lib/dynscreen.p
 *                                 -> dynScreenUpdate					(for each tt_screenlist...)
 *                                      -> dynviewerUpdate
 * 
 *
 ********************************************************************************
 *
 * while this code makes no formal commitment to preserve backward
 * compatibility to version 9 it also does not go out of its way
 * break such compatibility - there are no longchar, int64, datetime
 * or other known v9 compatibility issuesin this module
 * 
 * IOW - it might work with v9.
 * 
 * OTOH the code does use PUBLISH and SUBSCRIBE which will not work
 * with v8.  so don't bother trying that.
 * 
 */

{lib/dynscreen.i}
{lib/protop.i}

define new global shared variable pt_first        as logical   no-undo.
define new global shared variable hDataset        as handle    no-undo.		/* ugly hack :( 					*/
define new global shared variable pt_browseFiller as character no-undo.

define variable dbgZ1 as logical no-undo initial no.				/* debug browse widget sizing issues			*/ 
define variable dbgZ2 as logical no-undo initial no.				/* debug browse widget display issues			*/ 
define variable dbgZ3 as logical no-undo initial no.				/* debug browse widget creation issues			*/ 
define variable dbgZ4 as logical no-undo initial no.				/* debug browse panel frame width issues		*/

define stream dbgstrm.
/* define stream inStrm. */							/* defined in lib/protop.i				*/

{lib/tt_screenlist.i}

define temp-table tt_screenElement no-undo
  field frameName     as character
  field elementName   as character
  field elementRow    as integer
  field elementCol    as integer
  field elementType   as character
  field frameHandle   as handle
  field elementHandle as handle
  field elementValue  as character
  field proType       as character
  field proFormat     as character
  field elementAlign  as character
  index scrFrame is unique primary
    frameName
    elementName
.

define new global shared temp-table tt_browseColumnList no-undo
  field browseName     as character
  field columnName     as character
  field bufferHandle   as handle
  field columnHandle   as handle
  field columnStart    as integer
  field columnWidth    as integer
  index brwCol is unique primary
    browseName
    columnName
.

define temp-table tt_browseDataGrid no-undo
  field browseName     as character
  field columnName     as character
  field bufferHandle   as handle
  field columnHandle   as handle
  field columnRow      as integer
  field columnStart    as integer
  field columnWidth    as integer
  index brwCol is unique primary
    browseName
    columnName
    columnRow
.

define variable ok as logical no-undo.

/* do these really need to be global?
 */

define variable ptWidth as integer no-undo initial 160.				/* how wide is the ProTop screen designed to be?	*/
define variable brXW    as integer no-undo.					/* extra browse width due to decoration			*/
define variable brXH    as integer no-undo.					/* extra browse height due to decoration		*/

define variable dcList  as character no-undo.					/* active data collector list				*/
define variable inLine  as character no-undo extent 16.				/* input line						*/

define new global shared variable brSP as integer no-undo.			/* browse spacing					*/

create widget-pool.

if session:window-system = "tty" then
  assign
    brXW = 0									/* left box + > + space + right box = 4			*/
    brXH = 2									/* top box + label + underline + bottom box = 4		*/
  .
 else
  assign
    brXW = 4									/* windows GUI apparently uses a smaller font?		*/
    brXH = 4
  .

if pt_browseFiller = "" or pt_browseFiller = ? then
  pt_browseFiller = os-getenv( "BROWSEFILLER" ).

if pt_browseFiller = ? then
  pt_browseFiller = "".

if length( pt_browseFiller ) > 1 then pt_browseFiller = substring( pt_browseFiller, 1, 1 ).

if dbgz2 = true then
  do:
    output stream dbgstrm to value( substitute( "&1/&2", pt_tmpdir, "dbgz2.debug" )).
    put stream dbgStrm unformatted today " " string( time, "hh:mm:ss" ) " detailed dynamic panel sizing is enabled" skip.
    output stream dbgstrm close.
  end.

/* get the list of data collectors that should be active even if they are not visible
 */

define variable pt3rtConfig as character no-undo.

run findCfgName( "pt3rt", input-output pt3rtConfig ).

input stream inStrm from value( pt3rtConfig ).
repeat:
  inLine = ?.
  import stream inStrm inLine.
  if inLine[1] begins "#" or inLine[1] = "" or inLine[1] = ? then next.
  dcList = dcList + "," + inLine[1]. 
end.
input stream inStrm close.

dcList = trim( dcList, "," ).


/*** Install self as a session super-procedure
 ***
 ***/

session:add-super-procedure( this-procedure ).

subscribe to "protop_show"      anywhere run-procedure "protop_show".
subscribe to "protop_display"   anywhere run-procedure "protop_display".
subscribe to "protop_getReqStr" anywhere run-procedure "protop_getReqStr".
subscribe to "resizeBrowse"     anywhere run-procedure "resizeBrowse".

return.


/** library of functions and procedures
 **
 **/



function setBrowseTitle returns handle ( f as handle, titleText as character ):

  define variable t as handle  no-undo.
  define variable w as integer no-undo.

  define variable z as character no-undo.

  if session:batch = no then
    w = min( f:width, current-window:width-chars ).
   else
    w = 160.

  create text t.

  assign
    t:frame        = f
    t:name         = f:name + "_title"
    t:dcolor       = getColorNum( "browseTitle" )
    t:fgcolor      = getColorNum( "browseTitle" )
  .

  if pt_browseFiller = "" then
    assign
      t:col          = ( w / 2 ) - ( length( titleText ) / 2 )
      t:format       = substitute( "x(&1)", length( titleText ))
      t:screen-value = titleText
    .
   else
    do:

      /* alternate version of the title with fill characters
       */

      z = fill( pt_browseFiller, integer(( w - ( length( titleText ) + 2 )) / 2 )).
      z = substitute( "&1 &2 &3", z, titleText, z ).

      assign
        t:col          = 1
        t:format       = substitute( "x(&1)", length( z ))
        t:screen-value = z
      .

    end.

  return t.

end.


function updateBrowseTitle returns handle ( t as handle, titleText as character ):

  define variable w as integer no-undo.

  if session:batch = no then
    w = min( t:frame:width, current-window:width-chars ).
   else
    w = 160.

  assign
    t:format       = substitute( "x(&1)", length( titleText ))
    t:col          = ( w / 2 ) - ( length( titleText ) / 2 )
    t:screen-value = titleText
    t:dcolor       = getColorNum( "browseTitle" )
    t:fgcolor      = getColorNum( "browseTitle" )
  .

  return t.

end.


function makeBrowseColumnHeader returns handle ( f as handle, wName as character, r as integer, c as integer, w as integer ):

  define variable t as handle no-undo.

  create text t.
  assign
    t:frame        = f
    t:name         = wName
    t:format       = substitute( "x(&1)", max( 1, w, length( wName )))
    t:row          = r
    t:col          = c
    t:dcolor       = getColorNum( "browseLabel" )
    t:fgcolor      = getColorNum( "browseLabel" )
    t:screen-value = wName
  .

  return t.

end.


function pruneGrid returns logical ( f as handle, b as handle, r as integer ):

  define variable i as integer no-undo.

  for each tt_browseDataGrid where tt_browseDataGrid.browseName = b:name and tt_browseDataGrid.columnRow > ( r + brXH ):

    delete object tt_browseDataGrid.columnHandle no-error.
    delete tt_browseDataGrid.

  end.

  return true.

end.


function gridRowDisplay returns logical ( f as handle, b as handle, r as integer ):

  define buffer tt_browseColumnList for tt_browseColumnList.

  define variable t as handle no-undo.

  define variable hiliteAttr as character no-undo.

  r = r + brxH.									/* r is the query result row#, add brxH for the header			*/

  if r > f:height-chars then return true.

  for each tt_browseColumnList where tt_browseColumnList.bufferHandle = b:

    if not valid-handle( tt_browseColumnList.bufferHandle ) then
      do:
        publish "logMsg" ( 3, "Bogus buffer handle in browseHeaderDisplay!" ).
        return false.
      end.

    if not valid-handle( tt_browseColumnList.bufferHandle:buffer-field( columnName ) ) then
      do:
        publish "logMsg" ( 3, "Bogus buffer field in rowDisplay!" ).
        return false.
      end.

    find tt_browseDataGrid
       where tt_browseDataGrid.browseName = tt_browseColumnList.browseName
         and tt_browseDataGrid.columnName = tt_browseColumnList.columnName
         and tt_browseDataGrid.columnRow  = r no-error.

    if not available ( tt_browseDataGrid ) then
      do:
        create tt_browseDataGrid.
        assign
          tt_browseDataGrid.browseName   = tt_browseColumnList.browseName
          tt_browseDataGrid.columnName   = tt_browseColumnList.columnName
          tt_browseDataGrid.columnRow    = r
          tt_browseDataGrid.columnStart  = tt_browseColumnList.columnStart
          tt_browseDataGrid.columnWidth  = tt_browseColumnList.columnWidth
          tt_browseDataGrid.bufferHandle = tt_browseColumnList.bufferHandle
        .
      end.

    if not valid-handle( tt_browseDataGrid.columnHandle ) then
      do:
        create text t.
        assign
          t:frame        = f
          t:name         = tt_browseColumnList.columnName + "_" + string( r, "999" )
          t:format       = tt_browseColumnList.columnHandle:format
          t:row          = r
          t:col          = tt_browseColumnList.columnHandle:column
          t:hidden  = yes							/* setting these to unhidden / visible also makes the parent frame visible, that is undesireable :(	*/
          t:visible = no							/* we fill hidden frames so that the email dump will have them 		*/
        .
        tt_browseDataGrid.columnHandle = t.
      end.

    tt_browseDataGrid.columnHandle:screen-value = string( b:buffer-field( tt_browseColumnList.columnName ):buffer-value, b:buffer-field( tt_browseColumnList.columnName ):format ).

    hiliteAttr = "".

    hilite(
      tt_browseDataGrid.bufferHandle,
      tt_browseDataGrid.columnName,
      tt_browseDataGrid.bufferHandle:buffer-field( columnName ):buffer-value,
      input-output hiliteAttr
    ).

    if hiliteAttr = "" then hiliteAttr = "data".

    if  valid-handle( tt_browseDataGrid.columnHandle ) then
      do:
        if session:window-system = "tty" then
          tt_browseDataGrid.columnHandle:dcolor  = getColorNum( hiliteAttr ).
         else
          tt_browseDataGrid.columnHandle:fgcolor = getColorNum( hiliteAttr ).
      end.

  end.

  return true.

end.


function trackElement returns logical ( frmH as handle, elemH as handle, eName as character, proType as character, proFormat as character, horAlign as character, r as integer, c as integer, elemType as character ):

  define buffer tt_screenElement for tt_screenElement.

  find tt_screenElement
    where elementName   = eName
      and frameHandle   = frmH 
    no-error.

  if not available( tt_screenElement ) then
    create tt_screenElement.

  assign
    tt_screenElement.frameName     = frmH:name
    tt_screenElement.elementName   = eName
    tt_screenElement.elementRow    = r
    tt_screenElement.elementCol    = c
    tt_screenElement.elementType   = elemType
    tt_screenElement.frameHandle   = frmH
    tt_screenElement.elementHandle = elemH
    tt_screenElement.proType       = proType
    tt_screenElement.proFormat     = proFormat
    tt_screenElement.elementAlign  = horAlign
  .

  return true.

end.


function addLabel returns logical ( f as handle, wName as character, r as integer, c as integer, w as integer ):

  define variable t as handle no-undo.

  create text t.
  assign
    t:frame        = f
    t:name         = wName
    t:format       = substitute( "x(&1)", max( 1, w, length( wName )))
    t:row          = r
    t:col          = c
    t:dcolor       = getColorNum( "label" )
    t:fgcolor      = getColorNum( "label" )
    t:screen-value = wName
    f:height-chars = max( r, f:height-chars )
  .

  trackElement( f, t, wName, "character", t:format, "right", r, c, "label" ).

  return true.

end.


function addDataElement returns logical ( f as handle, wName as character, r as integer, c as integer, w as integer, proType as character, proFormat as character, horAlign as character ):

  define variable t as handle no-undo.

  define variable pFmt as character no-undo.

  pFmt = proFormat.
  if pFmt = "" then pFmt = substitute( "x(&1)", max( 1, w )).

  create text t.
  assign
    t:frame        = f
    t:name         = wName
    t:width-chars  = w
    t:row          = r
    t:col          = c
    t:format       = substitute( "x(&1)", max( 1, w, length( wName )))
    t:screen-value = wName /* proType */ /* pFmt */				/* handy for debugging layout, use PAUSE in protop.p			*/
    f:height-chars = max( r, f:height-chars )
  .

  trackElement( f, t, wName, proType, pFmt, horAlign, r, c, "field" ).

  return true.

end.


procedure dynViewerInit:

  define input parameter f         as handle    no-undo.
  define input parameter dcName    as character no-undo.
  define input parameter ttName    as character no-undo.
  define input parameter dispOrder as integer   no-undo.
  define input parameter rMin      as integer   no-undo.
  define input parameter rMax      as integer   no-undo.
  define input parameter keyCmd    as character no-undo.

  define buffer tt_screenList for tt_screenList.

  define variable scrDef as handle no-undo.

  define variable q as handle  no-undo.
  define variable b as handle  no-undo.

  define variable i as integer no-undo.
  define variable r as integer no-undo.
  define variable c as integer no-undo.
  define variable w as integer no-undo.

  define variable result   as character no-undo.

  define variable proFmt   as character no-undo.
  define variable proType  as character no-undo.
  define variable horAlign as character no-undo.

  define variable fName as character no-undo.
  define variable cText as character no-undo.

  define variable bx as handle  no-undo.

  define variable fillChar as character no-undo initial " ".			/* "." is useful for layout debugging...				*/

  define variable cut1 as integer no-undo.
  define variable cut2 as integer no-undo.

  create temp-table scrdef.

  run getConfig( substitute( "&1.json", lc( dcName )), output result ).

  if result begins substitute( '~{"tt_&1":[', "scrdef" /* dcName */ ) then
    do:

      scrdef:add-new-field( "ctrlName",   "character" ).			/* this could be converted to a static TT...				*/
      scrdef:add-new-field( "ctrlTxt",    "character" ).
      scrdef:add-new-field( "ctrlCol",    "integer" ).
      scrdef:add-new-field( "ctrlRow",    "integer" ).
      scrdef:add-new-field( "ctrlWid",    "integer" ).
      scrdef:add-new-field( "proType",    "character" ).
      scrdef:add-new-field( "proFmt",     "character" ).
      scrdef:add-new-field( "HAlign",     "character" ).

      scrdef:temp-table-prepare( "tt_scrdef" /* dcName */ ).			/* to use dcName requires change to backend tools/mk*			*/

      result = replace( result, "~n", "" ).					/* pretty printed json is too challenging for json2tt			*/
      result = trim( substring( result, index( result, "[" ) + 1 ), "~{~}]" ).	/* and we just want the rows - none of the wrapper			*/

      run json2tt( string( result ), scrdef:default-buffer-handle ).

    end.

  assign
    b      = scrDef:default-buffer-handle
    f:name = ttName
    f:row  = ( if dcName = "DBId" then 2 else 4 )				/* the dbid header is on line 2, other viewers on line 4		*/
  .

  create query q.

  find tt_screenList where tt_screenList.ttName = ttName no-error.
  if not available( tt_screenList ) then
    do:
      create tt_screenList.
      assign
        tt_screenList.displayOrder  = dispOrder
        tt_screenList.screenName    = f:private-data
        tt_screenList.ttName        = ttName
        tt_screenList.dcName        = dcName
        tt_screenList.ptCmd         = keyCmd
        tt_screenList.frameHandle   = f
        tt_screenList.bufferHandle  = ?
        tt_screenList.queryHandle   = q
        tt_screenList.screenType    = "viewer"
        tt_screenList.screenVisible = ( dcName = "DBId" )			/* initially only "dbid" is automatically visible			*/
        tt_screenList.screenRow     = f:row
        tt_screenList.panelWidth    = f:width-chars
      .
    end.

  /* when we initialize the viewer fill in fields with the field name
   */

  q:set-buffers( b ).
  q:query-prepare( "for each " + scrDef:name ).
  q:query-open.

  q:get-next().

  do while q:query-off-end = false:

    i = i + 1.

    assign
      fName = b:buffer-field( "CtrlName"  ):buffer-value()
      cText = b:buffer-field( "CtrlTxt"   ):buffer-value()
    .

    assign
      c     = b:buffer-field( "CtrlCol" ):buffer-value()
      r     = b:buffer-field( "CtrlRow" ):buffer-value()
      w     = b:buffer-field( "CtrlWid" ):buffer-value()
    .

    assign
      proType  = b:buffer-field( "proType" ):buffer-value()
      horAlign = b:buffer-field( "HAlign"  ):buffer-value()
      proFmt   = b:buffer-field( "proFmt"  ):buffer-value()
    .

    assign
      r = max( 1, r )
      c = max( 1, c )
    .

    if length( cText ) < w then cText = fill( fillChar, w - length( cText )) + cText.

    if       horAlign = "R" then horAlign = "Right".
     else if horAlign = "C" then horAlign = "Center".
     else if horAlign = "L" then horAlign = "Left".

    if r-index( fName, "_l" ) = length( fName ) - 1 then
      addLabel( f, cText, r, c, w ). 
     else
      addDataElement( f, fName, r, c, w, proType, proFmt, horAlign ).

    q:get-next().

  end.

  delete object scrDef.

  /* do not delete object q -- it is persisted in tt_screenList for later usage */

  return. /* true. */

end.

procedure dynBrowserInit:

  define input parameter f         as handle    no-undo.
  define input parameter dcName    as character no-undo.
  define input parameter ttName    as character no-undo.
  define input parameter dispOrder as integer   no-undo.
  define input parameter rMin      as integer   no-undo.
  define input parameter rMax      as integer   no-undo.
  define input parameter keyCmd    as character no-undo.

  define buffer tt_screenList for tt_screenList.

  define variable q as handle  no-undo.

  find tt_screenList where tt_screenList.ttName = ttName no-error.
  if not available( tt_screenList ) then
    do:

      create query q.

      create tt_screenList.
      assign
        tt_screenList.displayOrder  = dispOrder
        tt_screenList.screenName    = f:private-data
        tt_screenList.ttName        = ttName
        tt_screenList.dcName        = dcName
        tt_screenList.ptCmd         = keyCmd
        tt_screenList.frameHandle   = f
        tt_screenList.queryHandle   = q						/* browsers need a query						*/
        tt_screenList.screenType    = "browser"
        tt_screenList.screenVisible = no
        tt_screenList.minRows       = rMin
        tt_screenList.maxRows       = rMax
        tt_screenList.panelWidth    = f:width-chars
      .
    end.

  /* do not delete object q -- it is persisted in tt_screenList for later usage */

  return. /* true. */

end.


define variable oldLabelColor as integer no-undo initial ?.

procedure dynViewerUpdate:

  define input parameter q as handle no-undo.
  define input parameter f as handle no-undo.
  define input parameter b as handle no-undo.

  define buffer tt_screenElement for tt_screenElement.

  define variable i as integer no-undo.						/* tables								*/
  define variable j as integer no-undo.						/* fields								*/
  define variable k as integer no-undo.						/* fields in a buffer							*/

  define variable r as integer no-undo.						/* data rows								*/
  define variable c as integer no-undo.						/* columns								*/
  define variable h as integer no-undo.						/* height								*/
  define variable w as integer no-undo.						/* width								*/

  define variable ok as logical no-undo.
  define variable x  as handle  no-undo.

  define variable bufList    as character no-undo.
  define variable hiliteAttr as character no-undo.
  define variable labelColor as integer   no-undo.

  labelColor = getColorNum( "label" ). 

  if labelColor <> oldLabelColor then            
    do:
      for each tt_screenElement where tt_screenElement.elementType = "label":
        if session:window-system = "tty" then
          tt_screenElement.elementHandle:dcolor = labelColor.
         else
          tt_screenElement.elementHandle:fgcolor = labelColor.
      end.
      oldLabelColor = labelColor.
    end.

  q:get-first().

  do while not q:query-off-end:

    r = r + 1.

    do j = 1 to b:num-fields:

      c = c + 1.
      x = b:buffer-field( j ).

      find tt_screenElement where frameName = b:name and elementName = x:name no-error.

      /* update a screen element
       */

      if available( tt_screenElement ) then
        do:

          h = max( h, tt_screenElement.elementRow ).

          hiliteAttr = "".

          if f:name = "tt_dbId" then hiliteAttr = "header".

          /* hard coded data elements being used as labels
           */

          if lookup( elementName, "zUsrExpLH,zusrExpCS,usrLabel1,usrLabel2,usrLabel3,usrLabel4,usrLabel5,usrLabel6,usrLabel7,usrLabel8" ) > 0 then
            hiliteAttr = "label".

          /* should we alert on this value?
           */

          hilite( b, elementName, x:buffer-value, input-output hiliteAttr ).

          if hiliteAttr = "" then hiliteAttr = "data".

          if valid-handle( tt_screenElement.elementHandle ) then
            if session:window-system = "tty" then
              tt_screenElement.elementHandle:dcolor  = getColorNum( hiliteAttr ).
             else
              tt_screenElement.elementHandle:fgcolor = getColorNum( hiliteAttr ).

          case tt_screenElement.proType:

           when "integer" then tt_screenElement.elementHandle:screen-value = string( integer( x:buffer-value ), tt_screenElement.proFormat ) no-error.
           when "decimal" then tt_screenElement.elementHandle:screen-value = string( decimal( x:buffer-value ), tt_screenElement.proFormat ) no-error.
           otherwise
             do:
               if tt_screenElement.elementAlign = "Right" then
                 tt_screenElement.elementHandle:screen-value =
                   fill( " ", integer( tt_screenElement.elementHandle:width-chars ) - length( trim( x:buffer-value ))) +
                   trim( x:buffer-value ).
                else if tt_screenElement.elementAlign = "Center" then
                 tt_screenElement.elementHandle:screen-value =
                   fill( " ", integer(( tt_screenElement.elementHandle:width-chars - length( trim( x:buffer-value ))) / 2 )) +
                   trim( x:buffer-value ).
                else /* "left" */
                 tt_screenElement.elementHandle:screen-value = trim( x:buffer-value ).
             end.

          end.

          tt_screenElement.elementValue = tt_screenElement.elementHandle:screen-value.

          if error-status:num-messages > 0 then
            do:
              publish "logMsg" ( 4, substitute( "Error rendering: &1 &2 &3 &4", x:name, x:buffer-value, tt_screenElement.proType, tt_screenElement.proFormat )).
            end.

          /* I really should be ashamed of myself for hard-coding this dbidLogNm assignment :(  To say nothing of the shared variable...
           */

          if elementName = "dbidLogNm" then ptDBName = x:buffer-value.

          /* if something *really* hinky is going on with displaying
           * screen elements you might want to enable this
           */

          if false /* elementName = "whatever" */ then
            do:

              message
                skip(1)
                " elementName:"  elementName   skip
                "  elementRow:"  elementRow    skip
                "  elementCol:"  elementCol    skip
                "elementWidth:"  length( elementValue ) skip
                "elementAlign:"  elementAlign  skip
                "   proFormat:"  proFormat     skip
                "elementValue:"  substitute( "[&1]", elementValue ) skip
                skip(1)
               view-as
                alert-box information
                title "dbId info"
              .

            end.

        end.

    end.

    q:get-next().								/* why would a viewer ever have a NEXT? I suppose nothing is hurt by it but it seems odd	*/

    leave.									/* just in case there *is* a NEXT					*/

  end.

  /* if the screen has been shrunk to be shorter than the viewer
   * or stretched to be taller adjust the height
   */

  if h > f:height then
    do:
      if h > ( 1 + screen-lines - f:row ) then
        f:height = min( h, ( 1 + screen-lines - f:row )).
    end.

  return.

end.


procedure getSortField:

  define input parameter b as handle no-undo.

  define variable idxInfo   as character no-undo.				/* what field are we sorting on?					*/
  define variable zField    as character no-undo.				/* what field are we sorting on?					*/

  define variable i as integer no-undo.
  define variable j as integer no-undo.

  define buffer tt_screenList for tt_screenList.

  find tt_screenList where tt_screenList.ttName = b:name no-error.
  if not available( tt_screenList ) then
    do:
      /* message "getSortField() - that's odd, no tt_screenList for:" b:name view-as alert-box information.	*/
      return.
    end.

  if tt_screenList.sortFieldList = "" then
    do j = 1 to b:num-fields:
      if b:buffer-field(j):extent = 0 and lookup( b:buffer-field( j ):name, tt_screenList.sortFieldList ) <= 0 then
        tt_screenList.sortFieldList = tt_screenList.sortFieldList + ( if tt_screenList.sortFieldList = "" then "" else "," ) + b:buffer-field( j ):name.
    end.

  i = 0.

  do while tt_screenList.sortBy = "":

    i = i + 1.
    idxInfo = b:index-information( i ).
    if idxInfo = ? then
      leave.
     else
      do:
        if integer( entry( 2, idxInfo )) = 1 and integer( entry( 3, idxInfo )) = 1 then		/* unique and primary					*/
          assign
            tt_screenList.sortBy  = entry( 5, idxInfo )
            tt_screenList.sortDir = ( if entry( 6, idxInfo ) = "0" then "Up" else "Down" )
          .
        if zField = "" and integer( entry( 3, idxInfo )) = 1 then		/* primary								*/
          assign
            tt_screenList.sortBy = entry( 5, idxInfo )
            tt_screenList.sortDir = ( if entry( 6, idxInfo ) = "0" then "Up" else "Down" )
          .
      end.

  end.

  if tt_screenList.sortBy = "stackNum" then tt_screenList.sortDir = "Up".

  /* message "getSortField:" b:name tt_screenList.sortBy. pause 1. */

  return.

end.


procedure dynBrowserConfig:

  define input parameter q as handle no-undo.
  define input parameter f as handle no-undo.
  define input parameter b as handle no-undo.

  define buffer tt_screenList for tt_screenList.
  define buffer tt_browseColumnList for tt_browseColumnList.

  define variable i as integer no-undo.
  define variable j as integer no-undo.

  define variable k as integer no-undo.						/* fields in a buffer							*/
  define variable c as integer no-undo.						/* column width								*/
  define variable s as integer no-undo.						/* starting column							*/

  define variable xlist as character no-undo.					/* fields to exclude from browsers					*/
  define variable zlist as character no-undo.					/* fields from buffer b to add to browse z				*/

  define variable bf as handle no-undo.

  find tt_screenList where tt_screenList.ttName = b:name no-error.
  if not available( tt_screenList ) then
    do:
      message "dynBrowserConfig() - that's odd, no tt_screenList for:" b:name view-as alert-box.
      pause.
      return.
    end.

  if not valid-handle( q ) then
    do:
      message "dynBrowserConfig() - invalid query associated with browse:" b:name " q:valid?" valid-handle( q ).
      pause.
      return.
    end.

  if b:name = "tt_txe" and pt_altWidth <> "" then
    return.

  /* this is deferred from protop_display until the first dynBrowserUpdate()/dynBrowserConfig() call
   * because the buffer and query are unknown at the time the browser panel is being created and,
   * therefore, add-columns-from() could not be used yet (but that was in the dynamic chui browse
   * days and is no longer the reason)
   */

  f = tt_screenList.frameHandle.
  s = f:column.

  find first tt_browseColumnList where browseName = b:name no-error.

  if not available( tt_browseColumnList ) then					/* we must need columns to be added!					*/
    do:

      xlist = "xid,xvalid".							/* fields that we never want to include					*/

      do k = 1 to b:num-fields:							/* walk through the fields						*/

        if lookup( b:buffer-field( k ):name, xlist ) > 0 then			/* exclude "xid" etc. from the width calculation			*/
          next.

        if b:buffer-field( k ):extent > 1 then					/* also suppress array fields						*/
          do:
            xlist = xlist + "," + b:buffer-field( k ):name.
            next.
          end.

        if b:buffer-field( k ):name begins "z" then				/* and suppress fields whose name beings with "z"			*/
          do:
            xlist = xlist + "," + b:buffer-field( k ):name.
            next.
          end.

        /* if a non-standard width has been selected filter fields out that do not have that width option set
         */

        if pt_altWidth <> "" then
          do:
            if b:buffer-field( k ):private-data <> pt_altWidth then
              do:
                xlist = xlist + "," + b:buffer-field( k ):name.
                next.
              end.
          end.

        c = ( max(								/* the label might be wider than the data format			*/
              b:buffer-field( k ):width-chars,
              length( b:buffer-field( k ):label )
            )).

        zlist = zlist + "," + b:buffer-field( k ):name.				/* add to the list of fields that are in the browse			*/

      end.

      xlist = trim( xlist, "," ).
      zlist = trim( zlist, "," ).

      do k = 1 to b:num-fields:							/* walk through the fields						*/

        /* set up a mapping table so that row-display triggers
         * can find the browse columns
         */

        bf = b:buffer-field( k ).						/* 10.0a barfs without this...						*/

        if lookup( bf:name, zlist ) <= 0 then
          next.

        c = ( max(								/* the label might be wider than the data format			*/
              b:buffer-field( k ):width-chars,
              length( b:buffer-field( k ):label )
            )).

        if b:buffer-field( k ):data-type <> "character" and length( b:buffer-field( k ):label ) < c then
          do:
            b:buffer-field( k ):label = fill( " ", c - length( b:buffer-field( k ):label )) + b:buffer-field( k ):label.
          end.

        find tt_browseColumnList where browseName = b:name and columnName = bf:name no-error.
        if not available( tt_browseColumnList ) then create tt_browseColumnList.
        assign
          tt_browseColumnList.columnHandle = makeBrowseColumnHeader( f, b:buffer-field( k ):label, 2, integer( s - f:column ) + 1, c )
          tt_browseColumnList.browseName   = b:name
          tt_browseColumnList.columnName   = b:buffer-field( k ):name
          tt_browseColumnList.columnStart  = s
          tt_browseColumnList.columnWidth  = c
          tt_browseColumnList.bufferHandle = b
        .

        if tt_screenList.ptCmd = "" then
          tt_screenList.titleHandle = setBrowseTitle( tt_screenList.frameHandle, tt_screenList.screenName ).
         else
          tt_screenList.titleHandle = setBrowseTitle( tt_screenList.frameHandle, substitute( "<&1> &2", tt_screenList.ptCmd, tt_screenList.screenName )).

        s = s + c + 1.

      end.

    end.

  return.

end.


procedure dynBrowserUpdate:

  define input parameter q as handle no-undo.
  define input parameter f as handle no-undo.
  define input parameter b as handle no-undo.

  define buffer tt_screenList for tt_screenList.
  define buffer tt_browseColumnList for tt_browseColumnList.

  define variable i as integer no-undo.						/* fields in a buffer							*/
  define variable k as integer no-undo.						/* fields in a buffer							*/

  define variable bf as handle no-undo.

  find tt_screenList where tt_screenList.ttName = b:name no-error.
  if not available( tt_screenList ) then
    do:
      message "dynBrowserUpdate() - that's odd, no tt_screenList for:" b:name view-as alert-box.
      pause.
      return.
    end.

  if not valid-handle( q ) then
    do:
      message "dynBrowserUpdate() - invalid query associated with browse:" b:name " q:valid?" valid-handle( q ).
      pause.
      return.
    end.

  run dynBrowserConfig( q, f, b ).

  /* update the title color
   */

  if valid-handle( tt_screenlist.titleHandle ) then
    tt_screenlist.titleHandle:dcolor = getColorNum( "browseTitle" ).

  /* show which column we're sorting the browse by
   */

  do k = 1 to b:num-fields:							/* walk through the fields						*/

    bf = b:buffer-field( k ).							/* 10.0a barfs without this...						*/

    find tt_browseColumnList where browseName = b:name and columnName = bf:name no-error.
    if not available( tt_browseColumnList ) or not valid-handle( tt_browseColumnList.columnHandle ) then next.

    if tt_browseColumnList.columnName = tt_screenList.sortBy then
      tt_browseColumnList.columnHandle:dcolor = ( if tt_screenList.sortDir = "up" then getColorNum( "sortUp" ) else getColorNum( "sortDown" )).
     else
      tt_browseColumnList.columnHandle:dcolor = getColorNum( "browseLabel" ).

  end.

  q:get-first().

  i = 0.
  do while q:query-off-end = false:

    i = i + 1.
    gridRowDisplay( f, b, i ).
    if i >= ( f:height-chars - 1 ) then leave.
    q:get-next().

  end.

  if q:query-off-end = true then
    pruneGrid( f, b, i ).

  if tt_screenList.screenVisible = yes then
    do:

      for each tt_browseDataGrid where tt_browseDataGrid.browseName = b:name:
         assign
           tt_browseDataGrid.columnHandle:hidden  = no
           tt_browseDataGrid.columnHandle:visible = yes
         .

      end.

    end.

  return.

end.


/* map ProDataSet to dynamic screen elements...
 */

function dynScreenUpdate returns logical:

  define buffer tt_screenList for tt_screenList.

  define variable q  as handle  no-undo.
  define variable b  as handle  no-undo.
  define variable f  as handle  no-undo.
  define variable t  as handle  no-undo.

  define variable i  as integer no-undo.					/* tables								*/
  define variable j  as integer no-undo.					/* fields								*/
  define variable k  as integer no-undo.					/* fields in a buffer							*/
  define variable w  as integer no-undo.					/* width								*/
  define variable ok as logical no-undo.

  define variable sortField    as character no-undo.
  define variable bufList   as character no-undo.

  /* match up the buffers in the ProDataSet with the available viewers & browsers
   */

  loop: do i = 1 to hDataSet:num-buffers:
    b = hDataSet:get-buffer-handle( i ).
    bufList = substitute( "&1&2&3", bufList, min( ",", bufList ), b:name ).
    find tt_screenList where tt_screenList.ttName = b:name no-error.
    if available( tt_screenList ) then
      tt_screenList.bufferHandle = b.
     else
      do:
        /* message "no tt_screenlist for" b:name view-as alert-box information.		*/
        next loop.
      end.
  end.

  /* associate *_Info with base data collector (*_Info data becomes the title)
   */

  do i = 1 to hDataSet:num-buffers:
    b = hDataSet:get-buffer-handle( i ) no-error.
    if valid-handle( b ) and b:name matches "*_Info" then
      do:
        find tt_screenList where tt_screenList.ttName = substring( b:name, 1, index( b:name, "_Info" ) - 1 ) no-error.
        if available tt_screenList /* and valid-handle( b:default-buffer-handle ) */ then
          do:
            b:find-first( "", no-lock ) no-error.
            if valid-handle( b ) and b:available and valid-handle( tt_screenList.titleHandle ) then
              do:
                if tt_screenList.ptCmd = "" then
                  tt_screenList.titleHandle = updateBrowseTitle( tt_screenList.titleHandle, b:buffer-field( "infoString" ):buffer-value ).
                 else
                  tt_screenList.titleHandle = updateBrowseTitle( tt_screenList.titleHandle, substitute( "<&1> &2", tt_screenList.ptCmd, b:buffer-field( "infoString" ):buffer-value )).
              end.
          end.
      end.
  end.

  /* hide any panels (of any sort) that should not be visible
   */

  for each tt_screenList where tt_screenList.screenVisible = no:

    assign
      f = tt_screenList.frameHandle
    .

    if valid-handle( f ) then
      assign
        tt_screenList.frameHandle:sensitive  = no
        tt_screenList.frameHandle:hidden     = yes
        tt_screenList.frameHandle:visible    = no
      .

  end.

  /* by waiting until dynScreenUpdate to actually create browse
   * panels we can avoid a lot of unnecessary re-sizing
   */

  /* get sort fields and default sort order for browse panels, set browse panels to hidden
   */

  for each tt_screenList where tt_screenList.screenVisible = yes and tt_screenList.screenType = "browser":

    if tt_screenList.sortBy = "" or tt_screenList.sortFieldList = "" then
      run getSortField ( tt_screenList.bufferHandle ).

    assign
      f = tt_screenList.frameHandle
    .

    if valid-handle( f ) then
      do:
        assign
          tt_screenList.frameHandle:sensitive  = no
          tt_screenList.frameHandle:hidden     = yes
          tt_screenList.frameHandle:visible    = no
        .
      end.

  end.

  /* process the viewers & browsers in the order that they are on the screen (by screen row)
   */

  bufList = "".

  for each tt_screenList
     where valid-handle( tt_screenList.bufferHandle ) = yes
       /* and tt_screenList.screenVisible = yes	*/				/* Update invisible screens so that the send email utility knows what is going on */
      by tt_screenList.displayOrder:

    f = tt_screenList.frameHandle.

    assign ok = true no-error.

    b = ?.
    if valid-handle( tt_screenList.bufferHandle ) then
      b = tt_screenList.bufferHandle.
     else
      do:
        message "dynScreenUpdate() - invalid buffer!" tt_screenList.ttName view-as alert-box information.
        next.
      end.

    if tt_screenList.screenType = "browser" then
      do:

        sortField = "".
        if tt_screenList.sortBy <> "" then
          sortField = substitute( "by &1 &2", tt_screenList.sortBy, ( if tt_screenList.sortDir = "Up" then "" else "descending" )).

        if dbgMode >= 9 and sortField = "" and tt_screenList.screenType = "browser" then
          do:
            message b:name "no sortBy!".
            pause.
          end.
 
        if b:name = "tt_pasoeInfo" then
          sortField = "by xid".

      end.

    assign ok = true no-error.

    q = tt_screenList.queryHandle.

    if valid-handle( q ) then							/* it would also be sensible to check if the proper buffer is linked to the query	*/
      do:
        q:query-close() no-error.
        q:set-buffers( b ) no-error.
        ok = q:query-prepare( "for each " + b:name + " " + sortField ) no-error.
        /* message "reused - query-prepare( for each" b:name sortField ")" ok. pause. */
      end.
     else									/* first time...							*/
      do:
        q:set-buffers( b ) no-error.
        ok = q:query-prepare( "for each " + b:name + " " + sortField ) no-error.
        /* message "new - query-prepare( for each" b:name sortField ")" ok. pause. */
      end.

    if ok = no or error-status:num-messages > 0 then				/* data collector did not return tt?					*/
      do:

        /* fires annoyingly when "users of" is active, not really an error
         *
         *       message "query-prepare( for each" b:name sortField ") failed".
         *       do i = 1 to error-status:num-messages:
         *         message "query-prepare( for each" b:name sortField ")" error-status:get-number( i ) error-status:get-message( i ) view-as alert-box information.
         *       end.
         */

        next.

      end.

    if not valid-handle( q ) then
      do:
        message "almost tried query-open() FOR EACH " b:name sortField " on an invalid query - something is very wrong." view-as alert-box information.
        next.
      end.
     else
      do:

        ok = q:query-open() no-error.

        if ok = no or error-status:num-messages > 0 then
          do:
            do i = 1 to error-status:num-messages:
              message "query-open()" b:name sortField error-status:get-number( i ) error-status:get-message( i ) view-as alert-box information.
            end.
            next.
          end.

        /* resultRows = 0 is ok but if doubts arise we can check it
         *
         *  /* if resultRows = 0 or ? do we have a problem?
         *   */
         *
         *  tt_screenList.resultRows = q:num-results no-error.
         *  if tt_screenList.screenType = "browser" and ( tt_screenList.resultRows = 0 or tt_screenList.resultRows = ? ) then
         *    do:
         *      /* message "query-open()" b:name sortField "has" tt_screenList.resultRows "rows". */	/* view-as alert-box. 		*/
         *      /* next. */										/* nothing to show		*/
         *    end.
         *
         */

        bufList = substitute( "&1&2&3", bufList, min( ",", bufList ), b:name ).

        if tt_screenList.screenType = "browser" then
          run dynBrowserUpdate( q, f, b ).
         else if tt_screenList.screenType = "viewer" then
          run dynViewerUpdate( q, f, b ).

      end.

  end. /* for each tt_screenList ... */

  /* make sure that panels that _should_ be visible are indeed visible
   */

  for each tt_screenList
     where valid-handle( tt_screenList.bufferHandle ) = yes
       and tt_screenList.screenVisible = yes
      by tt_screenList.displayOrder:

    f = tt_screenList.frameHandle.

    assign
      f:width-chars = tt_screenList.panelWidth
      f:hidden  = no
      f:visible = yes
    .

  end.

  return true.

end.


procedure protop_getReqStr:

  define input-output parameter reqString as character no-undo.

  define variable cmd as character no-undo.

  for each tt_screenList
     where tt_screenList.screenVisible = yes
        or lookup( tt_screenList.dcName, dcList ) > 0:

    cmd = cmd + ( if cmd > "" then "," else "" ) + tt_screenList.dcName.

  end.

  publish "logMsg" ( 9, substitute( "Command = &1", cmd )).

  reqString = cmd.

  return.

end.


/* make sure that certain groupings of viewers and browsers are properly associated with one another
 */

procedure protop_show:

  define input parameter dcName  as character no-undo.

  define buffer tt_screenList for tt_screenList.
  define buffer xtt_screenList for tt_screenList.

  define variable v as logical no-undo.

  find tt_screenList where tt_screenList.dcName = dcName no-error.
  
  if not available( tt_screenList ) then
    do:
      publish "logMsg" ( 3, substitute( 'unknown event: "&1"', dcName )).
      return.
    end.

  if lookup( dcName, "usrtblact,usridxact,usrstack,usrother" ) > 0 then
    do:
      find xtt_screenList where xtt_screenList.dcName = "UserInformation".
      if  xtt_screenList.screenVisible = no then return.
    end.

  if tt_screenList.screenType = "browser" then
    do:

      tt_screenList.screenVisible = not( tt_screenList.screenVisible ).

      /* link certain screens together using the visibility of tt_screenList as the shared visibilty
       */

      /* keep "latches", "resources" and "txe" bound together -- some odd things happen if "latches" is displayed first so
       * jump through some hoops and do "resources" 1st.  the 2nd arg to protop_display() is correct as "tt" NOT "xtt"!
       */

      if tt_screenList.dcName = "LatchActivity" then
        do:

          run protop_display(  tt_screenList.ttName, tt_screenList.screenVisible ).

          find xtt_screenList where xtt_screenList.dcName = "ResourceWaits".
          run protop_display( xtt_screenList.ttName, tt_screenList.screenVisible ).

          find xtt_screenList where xtt_screenList.dcName = "TXE".
          run protop_display( xtt_screenList.ttName, tt_screenList.screenVisible ).

        end.

       /* AppSrvStatus and AppSrvDetails should always be displayed together
        */

       else if tt_screenList.dcName = "AppSrvStatus" then
        do:
          find xtt_screenList where xtt_screenList.dcName = "AppSrvDetails".
          run protop_display( xtt_screenList.ttName, tt_screenList.screenVisible ).
          run protop_display(  tt_screenList.ttName, tt_screenList.screenVisible ).
        end.
       else if tt_screenList.dcName = "AppSrvDetails" then
        do:
          find xtt_screenList where xtt_screenList.dcName = "AppSrvStatus".
          run protop_display(  tt_screenList.ttName, tt_screenList.screenVisible ).
          run protop_display( xtt_screenList.ttName, tt_screenList.screenVisible ).
        end.

      /*** this was a reasonable thing to do before the blocker data was added to the notes for the blocked session
       *** maybe make it an option if anyone cares about it?
       *
       *     else if tt_screenList.dcName = "Blocked" then
       *      do:
       *        find xtt_screenList where xtt_screenList.dcName = "Blocker".
       *        run protop_display( xtt_screenList.ttName, tt_screenList.screenVisible ).
       *        run protop_display(  tt_screenList.ttName, tt_screenList.screenVisible ).
       *      end.
       *     else if tt_screenList.dcName = "Blocker" then
       *      do:
       *        find xtt_screenList where xtt_screenList.dcName = "Blocked".
       *        run protop_display(  tt_screenList.ttName, tt_screenList.screenVisible ).
       *        run protop_display( xtt_screenList.ttName, tt_screenList.screenVisible ).
       *      end.
       *
       ***/

       else
        run protop_display( tt_screenList.ttName, tt_screenList.screenVisible ).

    end.
   else if tt_screenList.screenType = "viewer" then
    do:

      for each xtt_screenList where xtt_screenList.screenType = "viewer" by xtt_screenList.displayOrder:

        if xtt_screenList.dcName = "DBId" then
          assign
            xtt_screenList.screenVisible        = yes
            xtt_screenList.frameHandle:hidden   = no
            xtt_screenList.frameHandle:visible  = yes
          .
         else if xtt_screenList.dcName = dcName then
          assign
            xtt_screenList.screenVisible        = not ( xtt_screenList.screenVisible )
            xtt_screenList.frameHandle:hidden   = not ( xtt_screenList.screenVisible )
            xtt_screenList.frameHandle:visible  = xtt_screenList.screenVisible
          .
         else
          assign
            xtt_screenList.screenVisible        = no
            xtt_screenList.frameHandle:hidden   = yes
            xtt_screenList.frameHandle:visible  = no
          .

      end.

      /* userInfo is special -- it hides all unrelated browsers and opens its own set of confederates
       */

      if tt_screenList.dcName = "UserInformation" then
        do:

          for each xtt_screenList where xtt_screenList.screenType = "browser":

            if lookup( xtt_screenList.dcName, "usrtblact,usridxact,usrstack,usrother" ) = 0 then
              assign
                xtt_screenList.screenVisible        = no			/* hide browsers that are not related to userInfo			*/
                xtt_screenList.frameHandle:hidden   = yes
                xtt_screenList.frameHandle:visible  = no
              .
             else
              assign
                xtt_screenList.screenVisible        = yes			/* reveal browsers that are related to userInfo				*/
                xtt_screenList.frameHandle:hidden   = no
                xtt_screenList.frameHandle:visible  = yes
              .

          end.

        end.
       else
        do:

          for each xtt_screenList						/* hide the userInfo associated browses					*/
             where xtt_screenList.screenType = "browser"
               and lookup( xtt_screenList.dcName, "usrtblact,usridxact,usrstack,usrother" ) > 0:

            assign
              xtt_screenList.screenVisible        = no				/* hide the userInfo associated browses					*/
              xtt_screenList.frameHandle:hidden   = yes
              xtt_screenList.frameHandle:visible  = no
            .

          end.

        end.

      if not available( tt_screenList ) then
        publish "logMsg" ( 1, substitute( "No tt_screenlist! &1", dcName )).

      run protop_display( tt_screenList.ttName, tt_screenList.screenVisible ).

    end.
   else
    do:
      publish "logMsg" ( 1, substitute( "Unknown screen type: &1", tt_screenList.screenType )).
    end.

  return.

end.


procedure protop_display:

  define input parameter ttName   as character no-undo.
  define input parameter scrState as logical   no-undo.

  define buffer tt_screenList for tt_screenList.

  define variable s as integer no-undo.
  define variable v as integer no-undo.
  define variable h as integer no-undo.
  define variable r as integer no-undo.
  define variable n as integer no-undo.
  define variable d as integer no-undo.
  define variable t as integer no-undo.

  define variable i as integer no-undo.
  define variable x as integer no-undo.
  define variable y as integer no-undo initial 0.
  define variable z as integer no-undo.

  define variable q as handle  no-undo.
  define variable b as handle  no-undo.
  define variable f as handle  no-undo.

  define variable browseStart as integer no-undo.
  define variable browseLines as integer no-undo.

  define variable latchHeight as integer no-undo.				/* latchHeight is the tallest of latches, resources & TXE		*/


  /* set the target panel to be (logically) visible or not
   * if "" is passed then there is no specific target so
   * this step gets skipped, which is ok; we then proceed
   * to recalculate the screen layout
   */

  find tt_screenList where tt_screenList.ttName = ttName no-error.
  if available tt_screenlist then tt_screenList.screenVisible = scrState.

  /* count the visible viewers and find out what
   * line browse panels can start on
   */

  v = 0.
  for each tt_screenList where tt_screenList.screenType = "viewer" and tt_screenList.screenVisible = yes by tt_screenList.displayOrder:
    browseStart = ( tt_screenList.frameHandle:row + tt_screenList.frameHandle:height-chars ).
    v = v + 1.
    /*** message v tt_screenList.dcName tt_screenList.frameHandle:row "+" tt_screenList.frameHandle:height-chars "=" browseStart. ***/
  end.

  /* count the browse panels that are supposed to be visible
   */

  for each tt_screenList where tt_screenList.screenType = "browser":

    if tt_screenList.screenVisible = yes then s = s + 1.
    if tt_screenList.screenVisible = yes and lookup( tt_screenList.dcName, "LatchActivity,ResourceWaits,TXE" ) > 0 then x = x + 1.

  end.

  if x >= 2 then s = ( s - ( x - 1 )).						/* if both "latches" and "resources" or "txe" are visible only count one of them	*/
										/* blocked & blocker are NOT beside each other - so no funny business	*/
  if s < 1 then									/* nothing is visible!							*/
    do:
      /* message "Nothing to see here!" view-as alert-box. */
      if pt_first = no and v = 1 then						/* v = 1 is the screen header						*/
        publish "protop_command" ( "?" ).					/* show the help screen							*/
      return.									/* nothing more to do!							*/
    end.

  if pt_first = yes then pt_first = no.						/* no longer the first time through					*/

  /* changes to active viewers (top panel) may result in browsers needing
   * to resize and/or move if the viewers are not all the same size
   */

  browseLines = ( screen-lines - browseStart ) + 1.				/* number of lines that browse panels can be displayed on		*/
  browseLines = browseLines - ( s * brSP ).					/* account for possible blank line between each browse			*/

  assign
    h = truncate(( browseLines / s ), 0 ) 					/* standard number of rows per box (not just data, includes headers)	*/
    r = browseLines - ( h * s )							/* remainder rows -- to dole out one at a time				*/
  .

  i = 0.
  d = 0.
  for each tt_screenList
     where tt_screenList.screenType = "browser"
       and tt_screenList.screenVisible = yes
       and tt_screenList.maxRows > 0						/* is this actually useful?						*/
       and tt_screenList.maxRows < h:						/* is this actually useful?						*/

    if dbgZ1 = true then
      message
        tt_screenList.dcName "donating" max( 0, ( h - tt_screenList.maxRows )) "unused rows." h tt_screenList.maxRows
       view-as
        alert-box information
      .

    assign
      d = d + 1									/* how many row donors are there?					*/
      i = i + max( 0, ( h - tt_screenList.maxRows ))				/* donate excess rows to the remainder pool				*/
    .

  end.

  r = r + i.									/* rows donated by boxes whose maxRows is less than h			*/

  t = ( s - d ).								/* takers = screens - donors						*/

  if dbgZ1 = true or h < ( brXH + 1 ) then					/* we must be able to display at least 1 row of data			*/
    do:

      /* fetch the straw that broke the camel's back
       */

      find tt_screenList where tt_screenList.ttName = ttName no-error.
      if not available tt_screenList then
        do:
          for each tt_screenList where tt_screenList.screenType = "browser":	/* no new browse, the screen size has been radically changed		*/
            tt_screenList.screenVisible = no.					/* so hide all browse panels 						*/
          end.
        end.
       else
        do:

          if dbgZ1 = true then							/* if the screen is getting crowded dump some useful debugging		*/
            message
               "There are too many browsers being displayed!   " skip
               skip(1)
               tt_screenList.screenName "will not be displayed." skip
               skip(1)
               "You must remove something prior to displaying  " skip
               "another browse.                                " skip
               skip(1)
               "screenLines:" screen-lines  "         " skip
               "browseStart:" browseStart   "         " skip
               "browseLines:" browseLines   "         " skip
               "   #visible:" s             "         " skip
               " new height:" h             "         " skip
               "      s * h:" ( s * h )     "         " skip
               "     takers:" t             "         " skip
               "     donors:" d             "         " skip
               "    donated:" i             "         " skip
               " extra rows:" r             "         " skip
               skip(1)
               "If you really want to display that much        " skip
               "stuff, restart with a taller window!           " skip
              view-as
               alert-box information
            .
           else
            message
               "There are too many browsers being displayed!   " skip
               skip(1)
               tt_screenList.screenName "will not be displayed." skip
               skip(1)
               "You must remove something prior to displaying  " skip
               "another browse.                                " skip
               skip(1)
               "If you really want to display that much        " skip
               "stuff, restart with a taller window!           " skip
              view-as
               alert-box information
            .

          /* or, just hide the panel that was one too many
           */

          tt_screenList.screenVisible = no.

        end.

      return.

    end.

  z = browseStart.

  for each tt_screenList
     where tt_screenList.screenType = "browser"
       and tt_screenList.screenVisible = yes
        by tt_screenList.displayOrder:

    /* size each browse -- try to use the screen space efficiently
     */

    if ( tt_screenList.maxRows > 0 ) and ( tt_screenList.maxRows < h ) then	/* see "is this actually useful?" note above				*/
      do:
        if dbgZ1 = true then
          message
            tt_screenList.dcName "maxRows:" tt_screenList.maxRows
            view-as alert-box information
          .
        n = tt_screenList.maxRows.						/* max *data* rows - doesn't include decorations			 */
      end.
     else
      do:
        n = h.									/* assume standard height						*/
        if tt_screenList.screenVisible = yes then				/* only donate rows from *visible* panels				*/
          do:
            if r > 0 and t > 0
               and lookup( tt_screenList.dcName, "resources,txe" ) = 0 then	/* only donate from latches						*/
              do:
                x = truncate(( r / t ), 0 ).					/* how big an extra chunk to take					*/
                if ( r / t ) > truncate(( r / t ), 0 ) then x = x + 1.		/* round up, favors earlier screen orders				*/
                if r >= x then							/* dole out remainder rows						*/
                  assign
                    n = n + x							/* take an additional chunk of rows					*/
                    r = r - x							/* decrement remainder rows						*/
                    t = t - 1							/* decrement the number of remaining "takers"				*/
                  .
                if dbgZ1 = true then
                  message
                    "donated" x "rows to" tt_screenList.dcName n r t
                    view-as alert-box information
                  .
              end.
          end.
      end.

    /* resources, latches, and txe -- column positioning is handled in lib/browse.p
     * all 3 panels need to be the same height because the are next to one another
     */

    if tt_screenList.dcName = "LatchActivity" then				/* latches is first in displayOrder and the tallest			*/
      latchHeight = n.

    if lookup( tt_screenList.dcName, "ResourceWaits,TXE" ) > 0 and latchHeight > 0 then
      n = latchHeight.

    /* at this point we know how big the browse panels need to be
     * and where to put them but we don't know the width until we
     * add the columns and that won't happen until we add the query
     * which doesn't happen in protop_display() because we have
     * not yet run the data collector if this is a new browse panel
     * so all of that is deferred until the next run of
     * dynBrowserUpdate()
     */

    assign ok = yes no-error.							/* reset error-status							*/

    assign
      f = tt_screenList.frameHandle
      f:width-chars  = min( f:width, current-window:width-chars )		/* the window width might change (in protop.p)				*/
      f:height-chars = n							/* change height *before* row!						*/
      f:row = ( if tt_screenList.screenVisible = no then 1 else ( z + brSP ))	/* invisible panels go on row 1						*/
      tt_screenList.screenRow = f:row						/* set the starting screen row number					*/
    no-error.

    /* on windows there are shenanigans that do not (currently)
     * need to be exposed but if these problems occur in tty
     * mode we want to know about it
     */

    if session:window-system = "tty" and dbgMode >= 1 and error-status:num-messages > 0 then
      do i = 1 to error-status:num-messages:
        publish "logMsg" ( 1, substitute( "Frame sizing error: &1 &2; row = &3 height = &4 width = &5 w:h = &6 w:w = &7", error-status:get-number(i), error-status:get-message(i), s, n, f:width-chars, current-window:height-chars, current-window:width-chars )).
      end.

    /* don't increment the panel row number for invisible panels
     */

    if tt_screenList.screenVisible = yes and lookup( tt_screenList.dcName, "LatchActivity,ResourceWaits" ) = 0 then
      z = z + n + brSP.								/* "resources" and "txe" on the same line as "latches"			*/

    /* rather than create the browse here we defer it until
     * dynScreenUpdate runs - in this way we avoid unneeded
     * screen refreshes.  especially when multiple browse
     * panels are being added at once
     */

  end.

  return.

end.


{ssg/sausage06.i}


procedure protopSort:								/* called from protop.p via the "^" command key				*/
  run lib/sortx.p.
end.


procedure resizeBrowse:								/* data collectors may call this when they feel it is appropriate	*/

  define input parameter zName    as character no-undo.
  define input parameter zMaxRows as integer   no-undo.

  zMaxRows = max( 2, zMaxRows ).

  define variable oldRows as integer no-undo.

  find tt_screenlist where tt_screenList.dcName = zName and tt_screenList.screenType = "browser" no-error.

  if tt_screenlist.screenVisible = no then return.

  if available tt_screenlist then
    assign
      oldRows = tt_screenList.maxRows
      tt_screenList.maxRows = max( 2, zMaxRows + 4 )
    .

  if available tt_screenlist and oldRows <> tt_screenList.maxRows then
    do:										/* kind of rinky-dink...						*/
      run protop_show ( zname ).						/* 1st time removes it							*/
      run protop_show ( zname ).						/* 2nd time re-adds it							*/
    end.

  return.

end.

/*** end ***/
