/* serverrpt.p
 *
 */

define new global shared variable pt_rptdir      as character no-undo initial ".".
define new global shared variable pt_shortname   as character no-undo.

define variable sql_param_name       as character no-undo format "x(30)"  label "SQL Parameter Name".
define variable sql_param_value      as character no-undo format "x(16)"  label "Value".
define variable sql_param_is-default as character no-undo format "yes/no" label "Default?".

define variable sum_mpb as integer no-undo.
define variable sum_cnx as integer no-undo.

define variable i as integer no-undo format ">>9".
define variable j as integer no-undo format ">>9".

define variable svt as integer no-undo.
define variable mpb as integer no-undo.
define variable ma  as integer no-undo.

define variable dashN  as integer no-undo.
define variable dashMn as integer no-undo.

define variable common  as character no-undo case-sensitive.
define variable rptOut as character no-undo.

define variable nastyGram as character no-undo format "x(90)".

define variable qh as handle no-undo.
define variable bh as handle no-undo.

define variable hasSrvBrkPID as logical no-undo.

define buffer broker for _servers.
define buffer srvrs  for _servers.

find _file no-lock where _file-name = "_servers" no-error.

if available _file then
  do:

    find dictdb._field no-lock where _field._file-recid = recid( _file ) and _field-name = "_Server-Broker-Pid" no-error.
    hasSrvBrkPID = available( _field ).

  end.

rptOut = substitute( "&1/&2.servers.rpt", pt_rptdir, max( pt_shortname, ldbname(1))).

find _file no-lock where _file._file-name = "_dbParams" no-error.

if not available _file or decimal( substring( proversion, 1, index( proversion, "." ) + 1)) < 11.5 then
  do:

    nastyGram = "  This functionality requires OpenEdge 11.5 or better with the _dbParams VST activated.  ".

    form
      skip(1)
      nastyGram skip
      skip(1)
     with
      frame oe115
      centered
      row 20
      overlay
      no-labels
      width 96
    .
    color display value( if opsys = "unix" then "RED" else "RED/WHITE" ) nastyGram with frame oe115.
    display nastyGram with frame oe115.
    pause.
    hide frame oe115 no-pause.

  end.
 else
  do:

    output to value( rptOut ).

    /***
     * 
     * Parameters common to both _dbParams and _Servers:
     * 
     * DbParams-Name                  DbParams-Desc
     * ─────────────────────────────
     * -ipver                         TCP/IP Version (-ipver): IPV4 (14268)
     * -maxport                       Maximum Port for Auto Servers (-maxport): 0 (5649)
     * -minport                       Minimum Port for Auto Servers (-minport): 0 (5648)
     * -ssj                           Database server side join support (-ssj): Enabled (19329)
     * -threadedServer                Threaded database server (-threadedServer): Enabled (19151)
     * -threadedServerStack           Thread stack size for threaded database servers (-threadedServer
     * -Ma                            Maximum Number of Clients Per Server (-Ma): 49 (4257)
     * -Mi                            Minimum Clients Per Server (-Mi): 0 (4259)
     * -Mm                            Message Buffer Size (-Mm): 0 (12818)
     * -Mp                            Servers per Protocol (-Mp): 0 (12819)
     * -Mpb                           Maximum Servers Per Broker (-Mpb): 0 (5647)
     * -N                             Network Type (-N):  (4263)
     * -PendConnTime                  Pending client connection timeout (-PendConnTime): 0 (10357)
     * -S                             Service Name (-S): 0 (4262)
     * -ServerType                    Broker server group support (-ServerType): BOTH (17804)
     * -SQLCursors                    SQL Server Max Open Cursors (-SQLCursors): 0 (10028)
     * -SQLLockWaitTimeout            Number of seconds for a SQL client to wait for a lock conflict
     * -SQLQuotedRowid                SQL rowid identifier (-SQLQuotedRowid): Non-standard (19134)
     * -SQLStack                      SQL Server Stack Size (-SQLStack): 0 (10026)
     * -SQLStmtCache                  SQL Server Statement Cache Size (-SQLStmtCache): 0 (10027)
     * -SQLTempBuff                   Size [1K byte units] of SQL Server temp table buffer (-SQLTempBu
     * -SQLTempDisk                   Size [1K byte units] of SQL Server temp table disk storage (-SQL
     * -SQLTempPageSize               Size [1K byte units] of SQL Server temp table data page (-SQLTem
     * -SQLTruncateTooLarge           Authorized data truncation (-SQLTruncateTooLarge): OFF (17955)
     * -SQLWidthUpdate                SQL Autonomous Schema Update (-SQLWidthUpdate): OFF (18156)
     * 
     ***/

    common = "-Mn,-n,-pollset,-prefetchDelay,-prefetchFactor,-prefetchNumRecs,-prefetchPriority,-Nmsgwait".

    put unformatted "Parameters common to all client/server connections:" skip.

    create buffer bh for table "_dbParams".
    create query qh.
    qh:set-buffers( bh ).
    qh:query-prepare( "for each _dbParams no-lock" ).
    qh:query-open.

    qh:get-next().

    do while qh:query-off-end = false:

      if lookup( bh:buffer-field( "_dbParams-Name" ):buffer-value(), common ) <= 0 then
        do:
          qh:get-next().
          next.
        end.

      display
        bh:buffer-field( "_dbParams-Name" ):buffer-value()                     format "x(30)"  label "Parameter Name"
        bh:buffer-field( "_dbParams-Value" ):buffer-value()                    format "x(16)"  label "Value"
        bh:buffer-field( "_dbParams-Desc" ):buffer-value()                     format "x(64)"  label "Description"
        logical( bh:buffer-field( "_dbParams-Is-Default" ):buffer-value() )    format "Yes/No" label "Is Default"
        logical( bh:buffer-field( "_dbParams-Is-Modifiable" ):buffer-value() ) format "Yes/No" label "Can Modify"
       with
        width 160
      .

      if bh:buffer-field( "_dbParams-Name" ):buffer-value() = "-n"  then dashN  = integer( bh:buffer-field( "_dbParams-Value" ):buffer-value() ) no-error.
      if bh:buffer-field( "_dbParams-Name" ):buffer-value() = "-Mn" then dashMn = integer( bh:buffer-field( "_dbParams-Value" ):buffer-value() ) no-error.

      qh:get-next().

    end.

    delete object bh.
    delete object qh.

    for each broker no-lock where broker._server-type = "login":

      find_serverType: do i = 1 to 64:
        if       buffer broker:handle:buffer-field( "_srvParam-name" ):buffer-value( i ) = "-ServerType" then svt = i.
         else if buffer broker:handle:buffer-field( "_srvParam-name" ):buffer-value( i ) = "-Mpb"        then mpb = i.
         else if buffer broker:handle:buffer-field( "_srvParam-name" ):buffer-value( i ) = "-Ma"         then ma  = i.
      end.

      i = 0.

      put unformatted skip(1).
      put unformatted fill( "=", 80 ) skip.
      put unformatted skip(1).

      if broker._server-portnum = 0 then
        put unformatted substitute( "Default &1 login broker on port &2 (unusable)", buffer broker:handle:buffer-field( "_srvParam-value" ):buffer-value( svt ), _server-portnum ) skip.
       else
        put unformatted substitute( "Remote &1 login broker on port &2", buffer broker:handle:buffer-field( "_srvParam-value" ):buffer-value( svt ), _server-portnum ) skip.

      j = svt.

      do i = svt to 1 by -1:

        j = j + 1.

        display

          /* i */
          buffer broker:handle:buffer-field( "_srvParam-name" ):buffer-value( i ) format "x(30)" label "Parameter Name"
          buffer broker:handle:buffer-field( "_srvParam-value" ):buffer-value( i )  label "Value"
          buffer broker:handle:buffer-field( "_srvParam-is-default" ):buffer-value( i ) label "Default?"

          space(9)

          /* j */
          buffer broker:handle:buffer-field( "_srvParam-name" ):buffer-value( j )                  when buffer broker:handle:buffer-field( "_srvParam-name" ):buffer-value( j )       <> ? and buffer broker:handle:buffer-field( "_srvParam-value" ):buffer-value( svt ) <> "ABL" @ sql_param_name
          buffer broker:handle:buffer-field( "_srvParam-value" ):buffer-value( j )                 when buffer broker:handle:buffer-field( "_srvParam-value" ):buffer-value( j )      <> ? and buffer broker:handle:buffer-field( "_srvParam-value" ):buffer-value( svt ) <> "ABL" @ sql_param_value
          logical( buffer broker:handle:buffer-field( "_srvParam-is-default" ):buffer-value( j ) ) when buffer broker:handle:buffer-field( "_srvParam-is-default" ):buffer-value( j ) <> ? and buffer broker:handle:buffer-field( "_srvParam-value" ):buffer-value( svt ) <> "ABL" @ sql_param_is-default

         with
          frame b
          width 132
          down
        .

        down 1 with frame b.

      end.

      sum_mpb = sum_mpb + integer( buffer broker:handle:buffer-field( "_srvParam-value" ):buffer-value( mpb )) + 1 no-error.
      sum_cnx = sum_cnx + ( integer( buffer broker:handle:buffer-field( "_srvParam-value" ):buffer-value( mpb )) + 1 ) * integer( buffer broker:handle:buffer-field( "_srvParam-value" ):buffer-value( ma )) no-error.

      for each _servers no-lock:

        if hasSrvBrkPID = no then next.

        if buffer _servers:handle:buffer-field( "_server-broker-pid" ):buffer-value <> buffer broker:handle:buffer-field( "_server-broker-pid" ):buffer-value then
          next.

        display
          _servers._server-id         label "Id"       format ">>>>>9"
          _servers._server-num        label "Server#"  format ">>>>>9"
          _servers._server-pid        label "PID"
          _servers._server-type       label "Type"
          _servers._server-portnum    label "Port"
          _servers._server-logins     label "Logins"   format "->>>>>>>>>>>>>>>>>>9"
         with
          frame c
          width 132
          down
        .

      end.

    end.

    put unformatted skip(1).
    put unformatted fill( "=", 80 ) skip.
    put unformatted skip(1).

    put unformatted "To fully support these remote servers:" skip.
    put unformatted substitute( "     -n should be at least: &1 (actual = &2)", sum_cnx + 5, dashN  ) skip.
    put unformatted substitute( "    -Mn should be at least: &1 (actual = &2)", sum_mpb + 1, dashMn ) skip.

    output close.

    run lib/fileviewer.p ( rptOut ).

  end.

return.

